-- Scorchio!
-- A multi-mob fire vulnerability manager
-- Ennui@Bloodhoof-EU
-- Email: ennuilg@gmail.com
-- Based on: ScorchWatch by Liccee

local SM = LibStub:GetLibrary("LibSharedMedia-3.0", true)
local PC = AceLibrary("PaintChips-2.0")
local L = AceLibrary("AceLocale-2.2"):new("Scorchio")

local SB = "Scorch_Bars"
local MOBNAME = 1
local REFRESHTIME = 2
local SCORCHSTACK = 3
local CASTS = 4

local playerName = UnitName("player")
local versionNumber = "0.62"
local testBars = false
local mageList = { }
local impScorch = nil
local lastTargetedBarname = nil

local options = { 
	handler = Scorchio,
	type = 'group',
	args = {
		baroptions = {
			type = 'group',
			name = L["Bar Options"],
			desc = L["Bar Options"],
			args = {
				anchor = {
					type = 'group',
					name = L["Anchor"],
					desc = L["Manage anchor and test bars"],
					order = 10,
					args = {
						anchortoggle = {
							type = 'execute',
							name = L["Toggle"],
							desc = L["Shows/hides the dragable anchor and test bars"],
							func = "ToggleAnchor",
							order = 101
						},
						anchorreset = {
							type = 'execute',
							name = L["Reset Position"],
							desc = L["Reset anchor position to center of screen"],
							func = "ResetAnchor",
							order = 102,
						},
					},
				},
				growup = {
					type = 'toggle',
					name = L["Grow Up"],
					desc = L["If checked, bar will stack above the anchor, not below"],
					get = function()
						return Scorchio.db.profile.growUp
					end,
					set = function(v)
						Scorchio.db.profile.growUp = not Scorchio.db.profile.growUp
						Scorchio:SetGrowUp()
					end
				},
				barwidth = {
					type = 'range',
					name = L["Width"],
					desc = L["Change the width of the bars"],
					order = 120,
					get = function()
						return Scorchio.db.profile.baroptions.width
					end,
					set = function(v)
						Scorchio.db.profile.baroptions.width = v
						if testBars then Scorchio:ShowTestBars() end                        
					end,
					min = 24,
					max = 512,
					step = 8,
				},
				barheight = {
					type = 'range',
					name = L["Height"],
					desc = L["Change the height of the bars"],
					order = 121,
					get = function()
						return Scorchio.db.profile.baroptions.height
					end,
					set = function(v)
						Scorchio.db.profile.baroptions.height = v
						if testBars then Scorchio:ShowTestBars() end
					end,
					min = 6,
					max = 64,
					step = 2,
				},
				scale = {
					type = 'range',
					name = L["Scale"],
					desc = L["Change the scale of the bars"],
					order = 122,
					min = 0.5,
					max = 2,
					get = function()
						return Scorchio.db.profile.baroptions.scale
					end,
					set = function(v)
						Scorchio.db.profile.baroptions.scale = v
						if testBars then Scorchio:ShowTestBars() end
					end
				},
				barcolour = {
					type = 'color',
					name = L["Bar Colour"],
					desc = L["Change bar colour"],
					order = 110,
					get = function()
						return unpack(Scorchio.db.profile.baroptions.fgcolours)
					end,
					set = function(r, g, b, a)
						Scorchio.db.profile.baroptions.fgcolours = {r, g, b, a}
						if testBars then Scorchio:ShowTestBars() end
					end,
					hasAlpha = true
				},
				backgroundcolour= {
					type = 'color',
					name = L["Background Colour"],
					desc = L["Change background colour"],
					order = 111,
					get = function()
						return unpack(Scorchio.db.profile.baroptions.bgcolours)
					end,
					set = function(r, g, b, a)
						Scorchio.db.profile.baroptions.bgcolours = {r, g, b, a}
						if testBars then Scorchio:ShowTestBars() end
					end,
					hasAlpha = true
				},
				targetbarcolour = {
					type = 'color',
					name = L["Target Bar Colour"],
					desc = L["Change bar colour for your target"],
					order = 112,
					get = function()
						return unpack(Scorchio.db.profile.baroptions.fgcolourstarget)
					end,
					set = function(r, g, b, a)
						Scorchio.db.profile.baroptions.fgcolourstarget = {r, g, b, a}
						if testBars then Scorchio:ShowTestBars() end
					end,
					hasAlpha = true
				},
				targetbackgroundcolour= {
					type = 'color',
					name = L["Target Background Colour"],
					desc = L["Change background colour for your target"],
					order = 113,
					get = function()
						return unpack(Scorchio.db.profile.baroptions.bgcolourstarget)
					end,
					set = function(r, g, b, a)
						Scorchio.db.profile.baroptions.bgcolourstarget = {r, g, b, a}
						if testBars then Scorchio:ShowTestBars() end
					end,
					hasAlpha = true
				},
				texture = {
					type = 'text',
					name = L["Texture"],
					desc = L["Change the texture of the bars"],
					order = 115,
					get = function() 
						return Scorchio.db.profile.baroptions.texture 
					end,
					set = function(v)
						Scorchio.db.profile.baroptions.texture = v
						if testBars then Scorchio:ShowTestBars() end
					end,
					validate = SM:List("statusbar")
				},                
				fadetime = {
					type = 'range',
					name = L["Fade Time"],
					desc = L["Change how long the bars remain after a vulnerability has faded. (-1 will stop them disappearing until you leave combat or the mob is rescorched)"],
					min = -1,
					max = 30,
					step = 1,
					get = function()
						return Scorchio.db.profile.baroptions.fadetime
					end,
					set = function(v)
						Scorchio.db.profile.baroptions.fadetime = v
					end
				},
				icon = {
					type = 'group',
					name = L["Icon"],
					desc = L["Show Scorchio! icon beside bars"],
					args = {
						iconshow = {
							type = 'toggle',
							name = L["Show"],
							desc = L["Show Scorchio! icon beside bars"],
							order = 100,
							get = function()
								return Scorchio.db.profile.baroptions.iconshow
							end,
							set = function(v)
								Scorchio.db.profile.baroptions.iconshow = not Scorchio.db.profile.baroptions.iconshow
								if testBars then Scorchio:ShowTestBars() end
							end
						},
						iconside = {
							type = 'text',
							name = L["Side"],
							disabled = function()
								return not Scorchio.db.profile.baroptions.iconshow
							end,
							desc = L["Show Scorchio! icon on left or right of bars"],
							order = 101,
							get = function() 
								return Scorchio.db.profile.baroptions.iconside 
							end,
							set = function(v)
								Scorchio.db.profile.baroptions.iconside = v
								if testBars then Scorchio:ShowTestBars() end
							end,
							validate = { ["LEFT"] = L["Left"], ["RIGHT"] = L["Right"] }
						},
					},
				},
			},
		},
		syncreceive = {
			type = 'group',
			name = L["Receive Data"],
			desc = L["Change sync data sources to listen to"],
			args = {
				scorchio = {
					type = 'toggle',
					name = L["Scorchio! Sync"],
					desc = L["Receive data from other Scorchio! users"],
					get = function()
						return Scorchio.db.profile.syncreceive.scorchio
					end,
					set = function()
						Scorchio.db.profile.syncreceive.scorchio = not Scorchio.db.profile.syncreceive.scorchio
					end
				},
				events = {
					type = 'toggle',
					name = L["Local Events"],
					desc = L["Receive data from combat log events within range"],
					get = function()
						return Scorchio.db.profile.syncreceive.events
					end,
					set = function()
						Scorchio.db.profile.syncreceive.events = not Scorchio.db.profile.syncreceive.events
					end
				},
			},
		},
		warningtime = {
			type = 'range',
			name = L["Warning Time"],
			desc = L["Length of time to be warned before Fire Vulnerability expires. (Set to 0 to disable)"],
			min = 0,
			max = 30,
			step = 1,
			get = function ()
				return Scorchio.db.profile.warningtime
			end,
			set = function(v)
				Scorchio.db.profile.warningtime = v
			end
		},
		sounds = {
			type = 'group',
			name = L["Sounds"],
			desc = L["Notification Sounds"],
			args = {
				on = {
					type = 'toggle',
					name = L["Toggle Sounds"],
					desc = L["Enable/Disable all sounds"],
					order = 110,
					get = function()
						return Scorchio.db.profile.sounds.on
					end,
					set = function()
						Scorchio.db.profile.sounds.on = not Scorchio.db.profile.sounds.on
					end
				},
				warningsound = {
					type = 'text',
					name = L["Warning Sound"],
					disabled = function()
						return not Scorchio.db.profile.sounds.on
					end,
					desc = L["Change the sound played when warning time is reached"],
					order = 111,
					get = function() 
						return Scorchio.db.profile.sounds.warningsound 
					end,
					set = function(v)
						Scorchio.db.profile.sounds.warningsound = v
						PlaySoundFile(SM:Fetch("sound",v))
					end,
					validate = SM:List("sound")
				},
				expiredsound = {
					type = 'text',
					name = L["Expired Sound"],
					disabled = function()
						return not Scorchio.db.profile.sounds.on
					end,
					desc = L["Change the sound played when Fire Vulnerability expires"],
					order = 112,
					get = function() 
						return Scorchio.db.profile.sounds.expiredsound 
					end,
					set = function(v)
						Scorchio.db.profile.sounds.expiredsound = v
						PlaySoundFile(SM:Fetch("sound",v))
					end,
					validate = SM:List("sound")
				},
			},
		},
		blacklist = {
			type = 'group',
			name = L["Blacklist"],
			desc = L["Blacklist mages who aren't 3/3 Imp Scorch"],
			args = {
				magelist = {
					type = 'text',
					name = L["Blacklist"],
					order = 100,
					desc = L["Blacklist mages who aren't 3/3 Imp Scorch. (Check mages to ignore their data)"],
					get = function(key)
						return Scorchio.db.profile.blacklist[key]
					end,
					set = function(key,value)
						Scorchio.db.profile.blacklist[key] = value
					end,
					multiToggle = true,
					validate = mageList
				},
				autoblacklist = {
					type = 'toggle',
					name = L["Automatic"],
					desc = L["Check to automatically blacklist mages in your party without 3/3 Imp Scorch"],
					order = 105,
					get = function()
						return Scorchio.db.profile.autoBlackList
					end,
					set = function()
						Scorchio.db.profile.autoBlackList = not Scorchio.db.profile.autoBlackList
						Scorchio:RebuildMageList()
					end
				},
				rebuild = {
					type = 'execute',
					name = L["Rebuild"],
					desc = L["Rebuild Mage List"],
					order = 110,
					func = "RebuildMageList"
				},
			},
		},
		debug = {
			type = 'group',
			name = "Debug",
			desc = "Debug Options",
			guiHidden = true,
			args = {
				duplicates = {
					type = 'toggle',
					name = "Duplicates",
					desc = "Notify of duplicate supression.",
					get = function()
						return Scorchio.db.profile.debug.duplicates
					end,
					set = function()
						Scorchio.db.profile.debug.duplicates = not Scorchio.db.profile.debug.duplicates
					end
				},
				sync = {
					type = 'toggle',
					name = "Sync Info",
					desc = "Display synchronisation debug information",
					get = function()
						return Scorchio.db.profile.debug.sync
					end,
					set = function()
						Scorchio.db.profile.debug.sync = not Scorchio.db.profile.debug.sync
					end                   
				},
				mob = {
					type = 'toggle',
					name = "Mob Info",
					desc = "Display mob debug information",
					get = function()
						return Scorchio.db.profile.debug.mob
					end,
					set = function()
						Scorchio.db.profile.debug.mob = not Scorchio.db.profile.debug.mob
					end
				},
				code = {
					type = 'toggle',
					name = "Code Info",
					desc = "Display code debug information",
					get = function()
						return Scorchio.db.profile.debug.code
					end,
					set = function()
						Scorchio.db.profile.debug.code = not Scorchio.db.profile.debug.code
					end
				},
				timers = {
					type = 'toggle',
					name = "Timers Info",
					desc = "Display timers debug information",
					get = function()
						return Scorchio.db.profile.debug.timers
					end,
					set = function()
						Scorchio.db.profile.debug.timers = not Scorchio.db.profile.debug.timers
					end
				},                  
			},
		},
	},
}

Scorchio = AceLibrary("AceAddon-2.0"):new("AceComm-2.0", "AceConsole-2.0", "AceDB-2.0", "AceEvent-2.0", "CandyBar-2.0", "FuBarPlugin-2.0", "Sink-1.0")

Scorchio:RegisterChatCommand("/scorchio", options)
Scorchio.OnMenuRequest = options

Scorchio:RegisterDB("ScorchioDB", "ScorchioDBPC")
Scorchio:RegisterDefaults("profile", {
	sink10OutputSink = "ChatFrame",
	baroptions = {
		barposition = {0, 50},
		p = "CENTER",
		rP = "CENTER",
		scale = 1.0,
		fadetime = -1,
		iconshow = true,
		iconside = "LEFT",
		fgcolours = { 0, 1, 0, 0.7 },
		bgcolours = { 1, 0, 0, 0.7 },
		fgcolourstarget = { 1, 0, 0, 0.7 },
		bgcolourstarget = { 1, 0, 0, 0.7 },
		texture = "Blizzard",
		width = 192,
		height = 16
	},
	syncreceive = {
		scorchio = true,
		events = true
	},
	warningtime = 6,
	debug = {
		duplicates = false,
		sync = false,
		mob = false,
		code = false,
		timers = false
	},
	sounds = {
		on = true,
		warningsound = "Bell",
		expiredsound = "Info"
	},
	blacklist = { },
	autoBlackList = false
}
)

Scorchio:SetCommPrefix("Scorchio")
Scorchio.commPrefix = "Scorchio"
Scorchio.hasIcon = "Interface\\AddOns\\Scorchio\\icons\\scorch"

local function Debug(class, msg)
	local text = "DEBUG: " .. msg
	if Scorchio.db.profile.debug[class] then
		Scorchio:Print(text)
	end
end

Scorchio.Debug = Debug

local function hasImpScorch()
	local nameTalent,_,_,_,currRank,_ = GetTalentInfo(2,10)
	if nameTalent == "Improved Scorch" and currRank == 3 then
		return true
	else
		return false
	end
end

Scorchio.hasImpScorch = hasImpScorch

function Scorchio:OnClick()
	self:ToggleAnchor()
end

function Scorchio:InitializeTables()
	scorchTable = { }
	casterTable = { }
	cleanTable = { }

	lastTargetedBarname = "0"
end

function Scorchio:OnInitialize()
end

function Scorchio:OnEnable()
	self:RegisterComm(self.commPrefix, "GROUP", "ReceiveMessage")
	self:RegisterComm(self.commPrefix, "WHISPER", "ReceiveMessage")
	self:SetDefaultCommPriority("ALERT")
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED")
	self:RegisterEvent("TIMER_COMPLETE", "TimerComplete")
	self.anchor = self:CreateAnchor()
	self:RegisterCandyBarGroup(SB)
	self:SetGrowUp()
	self:InitializeTables()

	if Scorchio.db.profile.blacklist[playerName] == nil then 
		Scorchio.db.profile.blacklist[playerName] = not hasImpScorch() 
		Debug("code", "Adding " .. tostring(playerName) .. " to the Blacklist as " .. tostring(Scorchio.db.profile.blacklist[playerName]))
	end

	self:RebuildMageList()
	castingScorch = false
end

function Scorchio:OnDisable()
end

function Scorchio:ShowTestBars()
		local name = "Test1"
		self:RegisterCandyBar(name, 30, name)
		self:RegisterCandyBarWithGroup(name, SB)
		self:SetCandyBarColor(name, unpack(Scorchio.db.profile.baroptions.fgcolourstarget))
		self:SetCandyBarBackgroundColor(name, unpack(Scorchio.db.profile.baroptions.bgcolourstarget))
		self:SetCandyBarFade(name, Scorchio.db.profile.baroptions.fadetime)
		if Scorchio.db.profile.baroptions.iconshow then
			self:SetCandyBarIcon(name,"Interface\\AddOns\\Scorchio\\icons\\scorchr1")
			self:SetCandyBarIconPosition(name,Scorchio.db.profile.baroptions.iconside)
		else
			self:SetCandyBarIcon(name, nil)
		end
		self:SetCandyBarWidth(name, Scorchio.db.profile.baroptions.width)
		self:SetCandyBarHeight(name, Scorchio.db.profile.baroptions.height)
		self:SetCandyBarScale(name, Scorchio.db.profile.baroptions.scale)
		self:SetCandyBarTexture(name, SM:Fetch("statusbar", Scorchio.db.profile.baroptions.texture))
		self:StartCandyBar(name)
		local name = "Test2"
		self:RegisterCandyBar(name, 30, name)
		self:RegisterCandyBarWithGroup(name, SB)
		self:SetCandyBarColor(name, unpack(Scorchio.db.profile.baroptions.fgcolours))
		self:SetCandyBarBackgroundColor(name, unpack(Scorchio.db.profile.baroptions.bgcolours))
		self:SetCandyBarFade(name, Scorchio.db.profile.baroptions.fadetime)
		if Scorchio.db.profile.baroptions.iconshow then
			self:SetCandyBarIcon(name,"Interface\\AddOns\\Scorchio\\icons\\scorchr3")
			self:SetCandyBarIconPosition(name,Scorchio.db.profile.baroptions.iconside)
		else
			self:SetCandyBarIcon(name, nil)
		end
		self:SetCandyBarWidth(name, Scorchio.db.profile.baroptions.width)
		self:SetCandyBarHeight(name, Scorchio.db.profile.baroptions.height)
		self:SetCandyBarScale(name, Scorchio.db.profile.baroptions.scale)
		self:SetCandyBarTexture(name, SM:Fetch("statusbar", Scorchio.db.profile.baroptions.texture))
		self:StartCandyBar(name)
		testBars = true
end

function Scorchio:HideTestBars()
	for c = 1,2 do
		local name = "Test" .. c
		self:UnregisterCandyBarWithGroup(name,SB)
		self:UnregisterCandyBar(name)
	end
	testBars = false
end

function Scorchio:ToggleAnchor()
	if self.anchor:IsVisible() then
		self.anchor:Hide()
		self:HideTestBars()
	else
		self.anchor:Show()
		self:ShowTestBars()
	end
	Debug("code","Anchor position: " .. tostring(Scorchio.db.profile.baroptions.barposition.x) .. ", " .. tostring(Scorchio.db.profile.baroptions.barposition.y))
end

function Scorchio:CreateAnchor()
	local f = CreateFrame("Button", nil, UIParent)
	f:SetWidth(160)
	f:SetHeight(25)

	f.owner = self

	if Scorchio.db.profile.baroptions.barposition.x and Scorchio.db.profile.baroptions.barposition.y and Scorchio.db.profile.baroptions.barposition.p and Scorchio.db.profile.baroptions.barposition.rP then
		f:ClearAllPoints()
		f:SetPoint(Scorchio.db.profile.baroptions.barposition.p, UIParent, Scorchio.db.profile.baroptions.barposition.rP, Scorchio.db.profile.baroptions.barposition.x, Scorchio.db.profile.baroptions.barposition.y)
	else
		f:SetPoint("CENTER", UIParent, "CENTER", 0, 50)
	end

	f:SetScript("OnDragStart", function() this:StartMoving() end )
	f:SetScript("OnDragStop",
	function()
		this:StopMovingOrSizing()
		local p, _, rP, x, y = this:GetPoint()
		Scorchio.db.profile.baroptions.barposition.x = math.floor(x)
		Scorchio.db.profile.baroptions.barposition.y = math.floor(y)
		Scorchio.db.profile.baroptions.barposition.p = p
		Scorchio.db.profile.baroptions.barposition.rP = rP
	end)

	f:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background",
	edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
	tile = false, tileSize = 16, edgeSize = 16,
	insets = { left = 5, right =5, top = 5, bottom = 5 }})
	f:SetBackdropColor(1,0,0,.6)
	f:SetMovable(true)
	f:RegisterForDrag("LeftButton")

	f.Text = f:CreateFontString(nil, "OVERLAY")
	f.Text:SetFontObject(GameFontNormalSmall)
	f.Text:ClearAllPoints()
	f.Text:SetTextColor(1, 1, 1, 1)
	f.Text:SetWidth(160)
	f.Text:SetHeight(25)
	f.Text:SetPoint("TOPLEFT", f, "TOPLEFT")
	f.Text:SetJustifyH("CENTER")
	f.Text:SetJustifyV("MIDDLE")
	f.Text:SetText("Scorchio! v" .. versionNumber)
	f:Hide()
	return f
end

function Scorchio:ResetAnchor()
	self.db.profile.baroptions.barposition.x = 0
	self.db.profile.baroptions.barposition.y = 50
	Scorchio.db.profile.baroptions.barposition.p = "CENTER"
	Scorchio.db.profile.baroptions.barposition.rP = "CENTER"
	self.anchor:ClearAllPoints()
	self.anchor:SetPoint("CENTER", UIParent, "CENTER", 0, 50)
	self.anchor:Show()
	self:ShowTestBars()
end

function Scorchio:SetGrowUp()
	if Scorchio.db.profile.growUp then
		self:SetCandyBarGroupPoint(SB, "BOTTOM", self.anchor, "TOP", -7, 0)
		self:SetCandyBarGroupGrowth(SB, true)
		if testBars then self:ShowTestBars() end
	else
		self:SetCandyBarGroupPoint(SB, "TOP", self.anchor, "BOTTOM", -7, 0)
		self:SetCandyBarGroupGrowth(SB, false)
		if testBars then self:ShowTestBars() end
	end
end

function Scorchio:CheckFireVuln(spellId, spellCount, mobname)
	if (spellId == 22959) then
        if spellCount >= 0 then
            self:SetStacks(mobname,spellCount)
            Debug("code", mobname .. " gained scorch " .. spellCount)
            self:CreateBar(scorchTable[mobname])
            self:SendCommMessage("GROUP", spellCount, mobname, "STAC") 
            Debug("sync", ">STAC: " .. tostring(spellCount) .. ", " .. tostring(mobname))
        end
    end 
end

function Scorchio:COMBAT_LOG_EVENT_UNFILTERED(timestamp, event, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
	local _, _, prefix, suffix = string.find(event, "(.-)_(.+)")
	local mobname 

	if dstName ~= nil then 
		mobname = dstName .. "_" .. tostring(dstGUID)
	else
        mobname = UnitName("target")
    end

	if (prefix == "SPELL") then
		local spellId, spellName, spellSchool, spellAuraType, spellCount, fvApplied
		spellCount = -1
		spellId = arg9
		if (suffix == "DAMAGE") then
			-- scorch2
			if (tonumber(spellId) == 27074) then
				if not Scorchio.db.profile.blacklist[srcName] then 
					self:SendCommMessage("GROUP", srcName, mobname, "SSYN") 
					Debug("sync",">SSYN: " .. tostring(srcName) .. ", " .. tostring(mobname))
					casterTable[srcName] = GetTime()
					self:UpdateTableScorch(mobname, GetTime())
					self:CreateBar(scorchTable[mobname])
				end
			end
			-- Code to handle Fire Vuln.
		elseif (suffix == "AURA_APPLIED") then
			spellCount = 1
			self:CheckFireVuln(arg9, spellCount, mobname)
		elseif (suffix == "AURA_APPLIED_DOSE") then
			self:CheckFireVuln(arg9, arg13, mobname)
		elseif (suffix == "AURA_REMOVED") then		
			spellCount = -1
			self:CheckFireVuln(arg9, spellCount, mobname)
		elseif (suffix == "AURA_REMOVED_DOSE") then						
			self:CheckFireVuln(arg9, arg13, mobname)			
		end 	
	elseif (prefix == "UNIT") then
		-- dead mob
		if dstName ~= nil and scorchTable[mobname] ~= nil then
			Debug("mob", tostring(mobname) .. " dies.")
			scorchTable[mobname] = nil
			self:UnregisterCandyBarWithGroup(mobname,SB)
			self:UnregisterCandyBar(mobname)
			Debug("sync", ">RDEA: " .. tostring(mobname))
			self:SendCommMessage("GROUP", mobname, mobname, "RDEA")
		end
	end
end

function Scorchio:ReceiveMessage(prefix, sender, distribution, arg1, arg2, arg3)
	if arg3 == nil or arg3 == "SSYN" then
		if not Scorchio.db.profile.syncreceive.scorchio then return end    
		local castTime = GetTime()
		local caster = arg1
		local mob = arg2
		Debug("sync", "<: " .. tostring(caster) .. ", " .. tostring(mob))
		Debug("code", tostring(caster) .. ": " .. tostring(casterTable[caster]) .. ", " .. tostring(castTime) .. " SYNC")
		if casterTable[caster] == nil or casterTable[caster] < castTime - 0.8 then
			if not Scorchio.db.profile.blacklist[caster] then
				casterTable[caster] = castTime
				self:UpdateTableScorch(mob, GetTime())
				self:CreateBar(scorchTable[mob])
			else
				Debug("sync", tostring(caster) .. " blacklisted. Ignoring event.")
			end
		else
			Debug("duplicates", "Duplicate event ignored (" .. tostring(caster) ..") SYNC")
		end

	elseif arg3 == "RDEA" then
		if not Scorchio.db.profile.syncreceive then return end
		local mob = arg1
		if scorchTable[mob] ~= nil then
			Debug("mob", tostring(mob) .. " dies.")
			scorchTable[mob] = nil
			if self:IsCandyBarRegistered(mob) then
				self:UnregisterCandyBarWithGroup(mob,SB)
				self:UnregisterCandyBar(mob)
			end 
		end

	elseif arg3 == "RTAL" then 
		Debug("sync", "<RTAL: " .. tostring(sender))
		self:SendCommMessage("WHISPER", sender, playerName, hasImpScorch(), "STAL")
	elseif arg3 == "STAL" then
		local caster = arg1
		local remImpScorch = arg2
		Debug("sync", "<STAL: " .. tostring(sender))
		Scorchio.db.profile.blacklist[caster] = not remImpScorch
		Debug("code", "Adding " .. tostring(caster) .. " to the Blacklist as " .. tostring(Scorchio.db.profile.blacklist[caster]))
	elseif arg3 == "STAC" then
		local stacks = arg1
		local mob = arg2
		self:SetStacks(arg2,arg1)
	end
end

function Scorchio:SetStacks(mob, stacks)
	if (tonumber(stacks) <= 0) then
		self:LostVuln(mob)
        if scorchTable[mob] == nil then return end
		if self:IsCandyBarRegistered(mob) then
			self:UnregisterCandyBarWithGroup(mob,SB)
			self:UnregisterCandyBar(mob)
		end 
	else
		if scorchTable[mob] == nil then
			scorchTable[mob] = { mob, GetTime(), stacks , 0}
		else
			scorchTable[mob][SCORCHSTACK] = stacks
		end	
	end 
	self:UpdateTableScorch(mob,GetTime());
end

function Scorchio:UpdateTableScorch(mob, time)
	if scorchTable[mob] == nil then                
		return
	end
	scorchTable[mob][REFRESHTIME] = time       	
	if Scorchio.db.profile.warningtime > 0 and scorchTable[mob][SCORCHSTACK] > 0 then Scorchio:ScheduleEvent("TIMER_COMPLETE",(30 - Scorchio.db.profile.warningtime), mob) end
end

function Scorchio:PLAYER_REGEN_ENABLED()
	for k,v in pairs(scorchTable) do
		self:UnregisterCandyBarWithGroup(k,SB)
		self:UnregisterCandyBar(k)
	end
	for k,v in pairs(cleanTable) do
		self:UnregisterCandyBarWithGroup(k,SB)
		self:UnregisterCandyBar(k)
	end
	self:InitializeTables()
end

function Scorchio:PARTY_MEMBERS_CHANGED()
	Scorchio:RebuildMageList()
end

function Scorchio:RebuildMageList()
	local tmp = { }
	local count = 0
	local ceiling = GetNumRaidMembers()
	local rop = "raid"
	if ceiling == 0 then 
		ceiling = GetNumPartyMembers()
		rop = "party"
		tmp = { playerName }
		count = count + 1
	end
	if ceiling == 0 then
		mageList = { playerName }
		options.args.blacklist.args.magelist.validate = mageList
		return
	end
	for i = 1, ceiling do
		local unit = rop..tostring(i)
		local _, unitclass = UnitClass(unit)
		local unitname, unitrealm
		if unitclass == "MAGE" then
			unitname, unitrealm = UnitName(unit)
			if unitrealm ~= nil then
				Debug("code",":"..unitname..":-:"..unitrealm..":")
				unitrealm = "-" .. unitrealm
				if unitrealm ~= "-" then unitname = unitname .. unitrealm end
			end
			table.insert(tmp, unitname)
			count = count + 1
		end
	end
	mageList = tmp
	for k,v in pairs(mageList) do
		if Scorchio.db.profile.blacklist[v] == nil then
			Scorchio.db.profile.blacklist[v] = false
			Debug("code", "New mage: " .. tostring(v))
			if Scorchio.db.profile.autoBlackList then
				-- self:SendCommMessage("WHISPER", v, playerName, hasImpScorch(), "RTAL")
				Debug("sync",">RTAL: " .. tostring(v))
			end
		end
	end
	Debug("code", tostring(count) .. " mages added.")
	options.args.blacklist.args.magelist.validate = mageList
end

function Scorchio:TimerComplete(mob)
	local timertime = GetTime()

	if scorchTable[mob] then
		Debug("timers", "TIMER: " ..tostring(mob) .. ", " .. tostring(scorchTable[mob][REFRESHTIME]) .. ", " .. tostring(timertime) .. ", " .. tostring(scorchTable[mob][REFRESHTIME] - timertime))
		if scorchTable[mob][REFRESHTIME] - timertime < (-30 + Scorchio.db.profile.warningtime) then
			if Scorchio.db.profile.sounds.on then
				PlaySoundFile(SM:Fetch("sound", Scorchio.db.profile.sounds.warningsound))
			end
			self:Pour(L["Rescorch "] .. self:GetName(mob) .. "!", 1, 0.647, 0)
		end
	end
end

function Scorchio:CreateBar(scorchTarget)
	if scorchTarget == nil then return end
	local mob = scorchTarget[MOBNAME]
	local elapsedTime = GetTime() - scorchTarget[REFRESHTIME]
	local remainingTime = math.floor (30 - elapsedTime)
	local scorchStack = scorchTarget[SCORCHSTACK]
	local text = "(x" .. scorchStack .. ")  " .. self:GetName(mob)
	local name = (mob)
	local ExpFun = function (mob)
		local test = mob
		self:LostVuln(test)
	end
	self:RegisterCandyBar(name, remainingTime, text)
	self:RegisterCandyBarWithGroup(name, SB)

	if UnitExists("target") and UnitName("target").."_"..UnitGUID("target") == name then
		self:SetCandyBarColor(name, "red")
	else
		self:SetCandyBarColor(name, unpack(Scorchio.db.profile.baroptions.fgcolours))
	end

	self:SetCandyBarBackgroundColor(name, unpack(Scorchio.db.profile.baroptions.bgcolours))
	self:SetCandyBarFade(name, Scorchio.db.profile.baroptions.fadetime)

	if Scorchio.db.profile.baroptions.iconshow then
		self:SetCandyBarIcon(name, "Interface\\AddOns\\scorchio\\icons\\scorchr" .. scorchStack)
		self:SetCandyBarIconPosition(name,Scorchio.db.profile.baroptions.iconside)
	else
		self:SetCandyBarIcon(name, nil)
	end

	self:SetCandyBarWidth(name, Scorchio.db.profile.baroptions.width)
	self:SetCandyBarHeight(name, Scorchio.db.profile.baroptions.height)
	self:SetCandyBarScale(name, Scorchio.db.profile.baroptions.scale)
	self:SetCandyBarTexture(name, SM:Fetch("statusbar", Scorchio.db.profile.baroptions.texture))
	self:SetCandyBarCompletion(name, ExpFun, mob)
	self:StartCandyBar(name)
end

function Scorchio:LostVuln(mob)
	if Scorchio.db.profile.sounds.on then
		PlaySoundFile(SM:Fetch("sound", Scorchio.db.profile.sounds.expiredsound))
	end
	self:Pour(L["Fire Vulnerability Expired on "] .. tostring(self:GetName(mob)), 1, 0, 0)
	cleanTable[mob] = true
	scorchTable[mob] = nil
end


function Scorchio:GetName(mob)
	local _, _, prefix, suffix = string.find(mob, "(.-)_(.+)")
	return prefix
end

function Scorchio:PLAYER_TARGET_CHANGED()
	if self:IsCandyBarRegistered(lastTargetedBarname) then
		self:SetCandyBarColor(lastTargetedBarname, unpack(Scorchio.db.profile.baroptions.fgcolours))
        self:SetCandyBarBackgroundColor(lastTargetedBarname, unpack(Scorchio.db.profile.baroptions.bgcolours))
	end

	if UnitExists("target") then
		lastTargetedBarname = UnitName("target") .. "_" .. tostring(UnitGUID("target"))

		if self:IsCandyBarRegistered(lastTargetedBarname) then
			self:SetCandyBarColor(lastTargetedBarname, unpack(Scorchio.db.profile.baroptions.fgcolourstarget))
            self:SetCandyBarBackgroundColor(lastTargetedBarname, unpack(Scorchio.db.profile.baroptions.bgcolourstarget))
		end
	else
		lastTargetedBarname = "0"
	end
end

